﻿<#
  .Synopsis
   Returns the Firmware and Secure Boot status.

  .Description
   Returns the Firmware status BIOS or UEFI and if UEFI
   the status of Secure Boot. 

  .Outputs
   Dialog box with status information 

  .Notes
     Programmer   : RetiredGeek (@askWoody.com) 
                    aka: ComputerMentor
     Created      : 02 Mar 2020
     Last Updated : 
     Current Vers : 1.0

  .Example
   [d:\path\]Get-PCFirmwareStatus.ps1

#>


# Two blank linea above required for Comment-Based Help


Function Get-BiosType {

<#
.Synopsis
   Determines underlying firmware (BIOS) type and returns an 
   integer indicating UEFI, Legacy BIOS or Unknown.
   Supported on Windows 8/Server 2012 or later
.DESCRIPTION
   This function uses a compiled Win32 API call to determine
   the underlying system firmware type.
.EXAMPLE
   If (Get-BiosType -eq 1) { # System is running UEFI firmware... }
.EXAMPLE
    Switch (Get-BiosType) {
        1       {"Legacy BIOS"}
        2       {"UEFI"}
        Default {"Unknown"}
    }
.OUTPUTS
   Integer indicating firmware type: 
   (1 = Legacy BIOS, 2 = UEFI, Other = Unknown)
.FUNCTIONALITY
   Determines underlying system firmware type
#>

 [OutputType([UInt32])]
 Param(
    [Parameter(Mandatory=$True)]
      [String] $CompName)

   $ATArgs = @{Language       = 'CSharp'
               ErrorAction    = 'SilentlyContinue' 
               TypeDefinition = @'
      
    using System;
    using System.Runtime.InteropServices;

    public class FirmwareType
    {
     [DllImport("kernel32.dll")]
     static extern bool GetFirmwareType(ref uint FirmwareType);

     public static uint GetFirmwareType()
     {
       uint firmwaretype = 0;
       if (GetFirmwareType(ref firmwaretype))
         return firmwaretype;
       else
         return 0; // API call failed, just return 'unknown'
     }
    }
'@
    } #End $ATArgs

    $ErrorActionPreference = 'Stop'
    Try {
      If (-not 
        [System.Management.Automation.PSTypeName][FirmwareType]) {
      }
    }
    Catch {  #Type does not Exist Create it!
      Add-Type @ATArgs
    }
    $ErrorActionPreference = 'SilentlyContinue'

    [FirmwareType]::GetFirmwareType()

}  #End  ---------------Function Get-BIOSTYPE ----------------

Clear-Host

Add-Type -AssemblyName "System.Windows.Forms"

$StatusMsg = {
  [Windows.Forms.MessageBox]::Show($Message,
                                   $WindowTitle, 
  [Windows.Forms.MessageBoxButtons]::OK , 
  [Windows.Forms.MessageBoxIcon]::$MsgIcon)}

  $WindowTitle = "Program Terminated:"
  $MsgIcon     = "Stop"

If ($PSVersionTable.PSVersion.Major -lt 4) {
  
  $Message = "You are running PowerShell version: " +
             "$($PSVersionTable.PSVersion)`n`n" + 
             "The program requires Version 4.0 to run.`n`n" +
             "Please upgrade your PowerShell."
  $Null = & $StatusMsg
  Exit
} #End If ($PSVersionTable...


$FT= (&{ 
 Switch (Get-BiosType -CompName "$Env:ComputerName") {
       1       {"Legacy BIOS"}
       2       {"UEFI"}
       Default {"Unknown"}
 }})

 $Message = "Firmware Type:     $FT`n"

If ($FT -eq 'UEFI') {
  Try {
    $GIArgs = @{Path = "HKLM:\SYSTEM\ControlSet001\" +   
                       "Control\SecureBoot\state"
                ErrorAction = 'Stop'}
    $UEFISB = get-item @GIArgs
    $UEFIStatus = 
      (&{If($UEFISB.GetValue('UEFISecureBootEnabled') -eq 1)
                             {"Enabled"} Else {"Disabled"}})
  }
  Catch { $UEFIStatus = "Not Suported" }   
       
  $Message += "Secure Boot UEFI: $UEFIStatus"

} #End If ($FT -eq 'UEFI')

$WindowTitle = "Firmware Status:"
$MsgIcon     = "Information"
$Null        = & $StatusMsg
